@@ -1,4 +1,3 @@
-import functools
 import ipaddress
 import itertools
 import logging
@@ -6,16 +5,19 @@ import os
 import sys
 import urllib.parse
 from logging import warning
-from typing import List, TextIO
+from typing import List, TextIO, Optional, Text, Tuple
 
 import act.api
 
+from cachetools import LRUCache
+
 from . import DEFAULT_VALIDATOR
 from .base import ActBase, Config, Origin
 from .fact import Fact, FactType, RelevantFactBindings, RelevantObjectBindings
 from .obj import Object, ObjectType
 from .schema import schema_doc
 
+_CACHE=LRUCache(maxsize=4096)
 
 def as_list(value):
     "Encapsulate value in list if value is not already a list/tuple"
@@ -26,8 +28,7 @@ def as_list(value):
     return value
 
 
-@functools.lru_cache(4096)
-def handle_fact(fact: Fact, output_format="json", output_filehandle: TextIO = sys.stdout) -> None:
+def handle_fact(fact: Fact, output_format="json", output_filehandle: Optional[TextIO] = None) -> None:
     """
     add fact if we configured act_baseurl - if not print fact
     This function has a lru cache with size 4096, so duplicates that
@@ -36,6 +37,24 @@ def handle_fact(fact: Fact, output_format="json", output_filehandle: TextIO = sy
     will use print to stdout if no file handle has been passed, otherwise
     it will write to the file handle specified
     """
+
+    # We do not set sys.stdout as default in the function signature
+    # because that breaks redirection in pytest
+    # https://github.com/pytest-dev/pytest/issues/2178
+
+    # We have switched from functools.lru_cache to a specific cache from cachetools
+    # because the functools.lru_cache decorator did not handle facts as expected
+
+    # This was recently handled, skip
+    if hash(fact) in _CACHE:
+        return
+
+    # Add to cache so it is skipped later
+    _CACHE[hash(fact)] = True
+
+    if not output_filehandle:
+        output_filehandle = sys.stdout
+
     if fact.config.act_baseurl:  # type: ignore
         fact.add()
     else:
@@ -119,7 +138,7 @@ Args:
     fact_value (str[] | str):     Only return Facts matching a specific value
     organization (str[] | str):   Only return Facts belonging to
                                   a specific Organization
-    source (str[] | str):         Only return Facts coming from a specific Source
+    origin (str[] | str):         Only return Facts coming from a specific Origin
     include_retracted (bool):     Include retracted Facts (default=False)
     before (timestamp):           Only return Facts added before a specific
                                   timestamp. Timestamp is on this format:
@@ -143,7 +162,7 @@ Returns ActResultSet of Facts.
                 "fact_type",
                 "fact_value",
                 "organization",
-                "source"])
+                "origin"])
 
         res = self.api_post("v1/fact/search", **params)
 
@@ -469,15 +488,23 @@ Returns created fact type, or exisiting fact type if it already exists.
         return fact_type
 
 
-def handle_uri(actapi: Act, uri: str, output_format="json") -> None:
+def handle_uri(actapi: Act, uri: str, output_format="json", output_filehandle: Optional[TextIO] = None) -> None:
     """Add all facts (componentOf, scheme, path, basename) from an URI to the platform
 
 Raises act.api.base.ValidationError if uri does not have scheme and address component.
 Make sure to catch this exception and not create other facts to an uri that fails this
 validation as it will most likely fail later when uploading the fact to the platform.
     """
+
+    # We do not set sys.stdout as default in the function signature
+    # because that breaks redirection in pytest
+    # https://github.com/pytest-dev/pytest/issues/2178
+
+    if not output_filehandle:
+        output_filehandle = sys.stdout
+
     for fact in uri_facts(actapi, uri):
-        handle_fact(fact, output_format=output_format)
+        handle_fact(fact, output_format=output_format, output_filehandle=output_filehandle)
 
 
 def uri_facts(actapi: Act, uri: str) -> List[Fact]:
@@ -506,7 +533,7 @@ Return: List of facts
         # Is address an ipv4 or ipv6?
         ip = ipaddress.ip_address(addr)
         addr_type = "ipv{}".format(ip.version)
-        addr = str(ip)
+        addr = ip.exploded
     except ValueError:
         addr_type = "fqdn"
 
@@ -544,3 +571,21 @@ Return: List of facts
             .destination("uri", uri))
 
     return facts
+
+
+def ip_obj(addr: Text) -> Tuple[Text, Text]:
+    """Return tuple of IP type and (expanded) IP address.
+
+Raises ValueError if addr is not valid IPv4 or IPv6 address
+"""
+
+    try:
+        # Is address an ipv4 or ipv6?
+        ip = ipaddress.ip_address(addr.strip())
+
+        if ip.version not in (4, 6):
+            raise ValueError('Unknown IP version: %s' % ip.version)
+
+        return ("ipv{}".format(ip.version), ip.exploded)
+    except ValueError:
+        raise ValueError('Invalid IP address: %s' % addr)
